// rzcounter.java - randomized counter
// Copyright (c) 1997 Roger Zeitel 
// NOT FOR RESALE
// <rzeitel@mars.superlink.net>
// http://mars.superlink.net/rzeitel

// Randomized flashing colorized counter

import java.awt.*;
import java.applet.Applet;

public class rzcounter extends Applet implements Runnable {
   static final int Digit=4;
   Thread mainThread=null;
   int width,height,charWidth,osW,osH,sleepTime;
   FontMetrics metrics;
   Image    offScrImage=null;
   Graphics	offScrGraphics=null;
   String strCount;
   Color bgColor;
   boolean changeNum=true;

   public final void init() {
      Font font;
      String s;
      int i;

      s=getParameter("bgColor");
		bgColor= (s==null) ? Color.black : parseColorString(s);
		this.setBackground(bgColor);
      
      s = getParameter("sleepTime");
      sleepTime= (s==null) ? 100: Integer.parseInt(s);

      s = getParameter("fontSize");
      i = (s==null) ? 20: Integer.parseInt(s);

      s=getParameter("fontName");
      if (s==null) s="TimesRoman";

      font = new Font(s,Font.BOLD,i);
      metrics = getFontMetrics(font);
/*
        System.out.println(
              " Leading="   + metrics.getLeading()
            + " Ascent="    + metrics.getAscent()
            + " Descent="   + metrics.getDescent()
            + " height="    + metrics.getheight()
        );
*/

      width = size().width;
      height = size().height;
      charWidth = width / Digit;
      osW = (charWidth  - metrics.charWidth('0')) / 2;
      osH = height - metrics.getDescent();

      resize(width,height);
      offScrImage = createImage(width,height);
      offScrGraphics = offScrImage.getGraphics();
      offScrGraphics.setFont(font);
   }

   public final void start() {
      if (mainThread == null) {
         mainThread = new Thread(this);
         mainThread.start();
      }
   }
    
   public final void stop() {
      if (mainThread != null) {
         mainThread.stop();
         mainThread = null;
      }
   }

   public final void run() {
      while (mainThread.isAlive()) {
         repaint();
         try {
            mainThread.sleep(sleepTime);
         } catch (InterruptedException e) {
            break;
         }
      }
   }


   public final void update(Graphics g) {
      if ( !changeNum)  return;

      // Clear screen
      offScrGraphics.setColor(bgColor);
      offScrGraphics.fillRect(0,0,width,height);

      // create random 4 digit number
      strCount=createNum(strCount,Digit);

      // draw colorized digits
      for (int i=0; i<Digit; i++) {
         offScrGraphics.setColor(new Color(intRand(256),intRand(256),intRand(256)));
         offScrGraphics.drawString(strCount.substring(i,i+1), charWidth*i+osW, osH);
      }
      paint(g);
   }

   public final void paint(Graphics g) {
      g.drawImage(offScrImage,0,0,this);
   }

   public final boolean mouseDown(Event e, int x, int y) {
      changeNum=(changeNum==true) ? false : true;
      return true;
   }

   private final int intRand(int n) {
      return (int)(Math.random()*n);
   }

   private final String createNum(String s, int numLen) {
      int i;
      s="";
      for (i=0; i<numLen; i++) s+=intRand(10)+"";
      return s;  
   }

	private final Color parseColorString(String colorString)  {
		if(colorString.length()==7){
			int R = Integer.valueOf(colorString.substring(1,3),16).intValue();
			int G = Integer.valueOf(colorString.substring(3,5),16).intValue();
			int B = Integer.valueOf(colorString.substring(5,7),16).intValue();
			return new Color(R,G,B);
		}
		else return Color.black;
	}
}

